#include <stdio.h>
#include <types.h>
#include <socket.h>
#include <string.h>
#include "ftp.h"
#include <in.h>
#include <netdb.h>
#include <unixio.h>
#ifdef VMS
#include <stdlib.h>
#define bcopy(src, dst, len) memcpy(dst, src, len)
#endif

#define EXIT(con, e, handler) \
  { con -> errno = e;  \
    if ( (con->func != NULL) && (e < 0) ) \
      (*(con->func)) (con, e, con->lastmsg); \
    handler (e);\
  }
   
#define QUIT 0
#define Ctrl(x) ((x) - '@')
#define CLEAR(x) memset ( &x , '\0' , sizeof x )
#define CUT(x) ((x)&0xff)

typedef int STATUS;
typedef char String[256];
typedef int Answers[];

typedef struct
{
  int sock;
  int datasock;
  FILE *datafile;
  char mode;
  int errno;
  char *lastmsg;
  int ch;
  STATUS (*func)();
  STATUS (*debug)();
} FTP;

#define FtpSetErrorHandler(con,f) con->func = f;
#define FtpSetDebugHandler(con,f) con->debug = f; 

#define FtpAccountM(ftp,acc)       FtpCommand(ftp,"ACCT %s",acc)
#define FtpUserM(ftp,user)         FtpCommand(ftp,"USER %s",user)
#define FtpPasswordM(ftp,pas)      FtpCommand(ftp,"PASS %s",pas)
#define FtpTypeM(ftp,type)         FtpCommand(ftp,"TYPE %s",type)
#define FtpChdirM(ftp,dir)         FtpCommand(ftp,"CWD %s",dir)
#define FtpMkdirM(ftp,dir)         FtpCommand(ftp,"MKD %s",dir)
#define FtpRmM(ftp,file)           FtpCommand(ftp,"DELE %s",file)
#define FtpGetFileM(ftp,file)      FtpCommand(ftp,"RETR %s",file)
#define FtpPutFileM(ftp,file)      FtpCommand(ftp,"STOR %s",file)
#define FtpAsciiM(ftp)             FtpTypeM(ftp,"A")
#define FtpBinaryM(ftp)            FtpTypeM(ftp,"I")

#define FtpOpenReadM(ftp,file)     FtpData(ftp,"RETR %s",file,"r")
#define FtpOpenWriteM(ftp,file)    FtpData(ftp,"STOR %s",file,"w")
#define FtpOpenAppendM(ftp,file)   FtpData(ftp,"APPE %s",file,"r")
#define FtpSOpenReadM(ftp,file)    FtpData(ftp,"RETR %s",file,(char *)0)
#define FtpSOpenWriteM(ftp,file)   FtpData(ftp,"STOR %s",file,(char *)0)
#define FtpSOpenAppendM(ftp,file)  FtpData(ftp,"APPE %s",file,(char *)0)

#define FtpDirectoryM(ftp,pat,out) FtpRetr(ftp,"LIST %s",pat,out)
#define FtpDirM(ftp,out)           FtpRetr(ftp,"LIST","",out)
#define FtpGetM(ftp,in,out)        FtpRetr(ftp,"RETR %s",in,out)

FILE
  *FtpFullOpen (char *file, char *mode);

STATUS
  FtpAbort (FTP *ftp),
  FtpAccount (FTP *con, char *acc),
  FtpAscii (FTP *con),
  FtpBinary (FTP *con),
  FtpBye (FTP *ftp),
  FtpChdir (FTP *con, char *dir),
  FtpClose (FTP *),
  FtpCommand (FTP *con, char *command, char *param),
  FtpConnect (FTP **con, char *hostname),
  FtpData (FTP *con, char *command, char *param, char *mode),
  FtpDebug (FTP *con),
  FtpDirectory (FTP *ftp, char *pat, char *out),
  FtpDir (FTP *ftp, char *out),
  FtpFullClose (FILE *),
  FtpGet (FTP *ftp, char *in, char *out),
  FtpGetFile (FTP *con, char *file),
  FtpGetString (FTP *con, char *str, int strlen, int *retlen),
  FtpGood(int n),
  FtpLogin (FTP **con, char *host, char *user, char *pass , char *acct),
  FtpMkdir (FTP *con, char *dir),
  FtpMove (FTP *con, char * old, char *new),
  FtpOpenAppend (FTP *ftp, char *file),
  FtpOpenDir(FTP *con, char * files),
  FtpOpenRead (FTP *ftp, char *file),
  FtpOpenWrite (FTP *ftp, char *file),
  FtpPassword (FTP *con, char *pas),
  FtpPort (FTP *con, int, int, int, int, int, int),
  FtpPut (FTP *con, char *inp, char *out),
  FtpPutFile (FTP *con, char *file),
  FtpRetr (FTP *con, char *command, char *inp, char *out),
  FtpRm (FTP *con, char *file),
  FtpSendMessage (FTP *con, char * Message),
  FtpSOpenAppend (FTP *ftp, char *file),
  FtpSOpenRead (FTP *ftp, char *file),
  FtpSOpenWrite (FTP *ftp, char *file),
  FtpType (FTP *con, char *type),
  FtpUser (FTP *con, char *user),
  FtpWrite (FTP *con, char c);

int
  FtpGetC (FTP *con),
  FtpGetMessage (FTP *con, char *Message),
  FtpNumber (char *Message),
  FtpSFullOpen (char *file, char *mode);

char 
  *FtpPwd (FTP *con),
  *FtpMessage (int Number);
